/*
 * Renderer 7. The MIT License.
 * Copyright (c) 2022 rlkraft@pnw.edu
 * See LICENSE for details.
*/

import renderer.scene.*;
import renderer.scene.util.ModelShading;
import renderer.models_L.Disk;
import renderer.models_L.SquareGrid;
import renderer.pipeline.*;
import renderer.framebuffer.*;

import java.awt.Color;
import javax.swing.JFrame;
import java.awt.BorderLayout;

/**



*/
@SuppressWarnings("serial")
public class Circle_v4_WindowToTheWorld extends Circle_v0_Abstract
{
   /**
      This constructor instantiates the Scene object
      and initializes it with appropriate geometry.
      Then this constructor instantiates the GUI.
   */
   public Circle_v4_WindowToTheWorld()
   {
      super("Renderer 7: Window To The World");

      // Remove the Model that's already in the Scene.
      scene.positionList.remove(0);

      // Create models for a 3x3 grid of circles and squares.
      final Position p1 = new Position(new Disk(1.0, 10, 40));
      final Position p2 = new Position(new SquareGrid(1.0, 15, 15));
      final Position p3 = new Position(new Disk(1.0, 10, 40));
      final Position p4 = new Position(new SquareGrid(1.0, 15, 15));
      final Position p5 = new Position(new Disk(1.0, 10, 40));
      final Position p6 = new Position(new SquareGrid(1.0, 15, 15));
      final Position p7 = new Position(new Disk(1.0, 10, 40));
      final Position p8 = new Position(new SquareGrid(1.0, 15, 15));
      final Position p9 = new Position(new Disk(1.0, 10, 40));

      // Translate eight of the nine models.
      p1.translation(-2,  2, 0);  // upper left-hand corner
      p2.translation( 0,  2, 0);  // center of top edge
      p3.translation( 2,  2, 0);  // upper right-hand corner
      p4.translation(-2,  0, 0);  // center of left edge
    //p5.translation( 0,  0, 0);  // the model at the center
      p6.translation( 2,  0, 0);  // center of right edge
      p7.translation(-2, -2, 0);  // lower left-hand corner
      p8.translation( 0, -2, 0);  // center of bottom edge
      p9.translation( 2, -2, 0);  // lower right-hand corner

      modelToRotate = p5;

      ModelShading.setRandomColor(p1.getModel());
      ModelShading.setRandomColor(p2.getModel());
      ModelShading.setRandomColor(p3.getModel());
      ModelShading.setRandomColor(p4.getModel());
      ModelShading.setColor(p5.getModel(), Color.red); // the model at the center
      ModelShading.setRandomColor(p6.getModel());
      ModelShading.setRandomColor(p7.getModel());
      ModelShading.setRandomColor(p8.getModel());
      ModelShading.setRandomColor(p9.getModel());

      // Add all the Models to the Scene.
      scene.addPosition(p1, p2, p3, p4, p5, p6, p7, p8, p9);

      print_help_message();
   }


   /**
      Get in one place the code to set up the viewport and the view volume.
   */
   @Override protected void setupViewing()
   {
      // Get the size of the FrameBuffer.
      final FrameBuffer fb = fbp.getFrameBuffer();
      final int wFB = fb.width;
      final int hFB = fb.height;

      // The viewport is the whole framebuffer.
      fb.setViewport(0, 0, wFB, hFB);

      if (mode == 1)
      {
         // 1. upper left-hand corner
         scene.getCamera().projOrtho(
                            -1.0,                      // left
                            -1.0 + (2.0 * wFB) / SIZE, // right
                             1.0 - (2.0 * hFB) / SIZE, // bottom
                             1.0,                      // top
                            -1.0);                     // near
      }
      else if (mode == 2)
      {
         // 2. center of the top edge
         scene.getCamera().projOrtho(
                            -wFB / (double)SIZE,       // left
                             wFB / (double)SIZE,       // right
                             1.0 - (2.0 * hFB) / SIZE, // bottom
                             1.0,                      // top
                            -1.0);                     // near
      }
      else if (mode == 3)
      {
         // 3. upper right-hand corner
         scene.getCamera().projOrtho(
                             1.0 - (2.0 * wFB) / SIZE, // left
                             1.0,                       // right
                             1.0 - (2.0 * hFB) / SIZE,  // bottom
                             1.0,                       // top
                            -1.0);                     // near
      }
      else if (mode == 4)
      {
         // 4. center of the right edge.
         scene.getCamera().projOrtho(
                             1.0 - (2.0 * wFB) / SIZE, // left
                             1.0,                       // right
                            -hFB / (double)SIZE,        // bottom
                             hFB / (double)SIZE,        // top
                            -1.0);                     // near
      }
      else if (mode == 5)
      {
         // 5. lower right-hand corner
         scene.getCamera().projOrtho(
                             1.0 - (2.0 * wFB) / SIZE, // left
                             1.0,                       // right
                            -1.0,                       // bottom
                            -1.0 + (2.0 * hFB) / SIZE,  // top
                            -1.0);                      // near
      }
      else if (mode == 6)
      {
         // 6. centered of the bottom edge
         scene.getCamera().projOrtho(
                            -wFB / (double)SIZE,        // left
                             wFB / (double)SIZE,        // right
                            -1.0,                       // bottom
                            -1.0 + (2.0 * hFB) / SIZE,  // top
                            -1.0);                      // near
      }
      else if (mode == 7)
      {
         // 7. lower left-hand corner
         scene.getCamera().projOrtho(
                            -1.0,                       // left
                            -1.0 + (2.0 * wFB) / SIZE,  // right
                            -1.0,                       // bottom
                            -1.0 + (2.0 * hFB) / SIZE,  // top
                            -1.0);                      // near
      }
      else if (mode == 8)
      {
         // 8. centered of the left edge
         scene.getCamera().projOrtho(
                            -1.0,                    // left
                            -1 + (2.0 * wFB) / SIZE, // right
                            -hFB / (double)SIZE,     // bottom
                             hFB / (double)SIZE,     // top
                            -1.0);                   // near
      }
      else if (mode == 9)
      {
         // 9. center of the framebuffer
         scene.getCamera().projOrtho(
                            -wFB / (double)SIZE,  // left
                             wFB / (double)SIZE,  // right
                            -hFB / (double)SIZE,  // bottom
                             hFB / (double)SIZE,  // top
                            -1.0);                // near
      }

      super.setupViewing();
   }


   /**
      Create an instance of this class which has
      the affect of creating the GUI application.
   */
   public static void main(String[] args)
   {
      // We need to call the program's constructor in the
      // Java GUI Event Dispatch Thread, otherwise we get a
      // race condition between the constructor (running in
      // the main() thread) and the very first ComponentEvent
      // (running in the EDT).
      javax.swing.SwingUtilities.invokeLater(
         () -> new Circle_v4_WindowToTheWorld()
      );
   }
}
